Package: iCOBRA
Type: Package
Title: Comparison and Visualization of Ranking and Assignment Methods
Version: 1.38.0
Date: 2025-07-17
Authors@R: c(person(given = "Charlotte", family = "Soneson", 
    role = c("aut", "cre"),
    email = "charlottesoneson@gmail.com",
    comment = c(ORCID = "0000-0003-3833-2169")))
Description: This package provides functions for calculation and visualization
    of performance metrics for evaluation of ranking and binary classification
    (assignment) methods. Various types of performance plots can be generated
    programmatically. The package also contains a shiny application for 
    interactive exploration of results.
License: GPL (>=2)
LazyData: TRUE
Collate: 'AllGenerics.R' 'COBRAData.R' 'COBRAPerformance.R'
        'COBRAPlot.R' 'calculate_performance.R' 'cobradata_example.R'
        'cobradata_example_sval.R' 'helpers_general.R' 'plot_methods.R'
        'printHead.R' 'shiny.R'
Depends: R (>= 4.4.0)
Imports: shiny (>= 0.9.1.9008), shinydashboard, shinyBS, reshape2,
        ggplot2 (>= 3.4.0), scales, ROCR, dplyr, DT, limma, methods,
        UpSetR, utils, rlang
Suggests: knitr, markdown, rmarkdown, testthat
VignetteBuilder: knitr
biocViews: Classification, Visualization
Encoding: UTF-8
RoxygenNote: 7.3.2
URL: https://github.com/csoneson/iCOBRA
BugReports: https://github.com/csoneson/iCOBRA/issues
git_url: https://git.bioconductor.org/packages/iCOBRA
git_branch: RELEASE_3_22
git_last_commit: 60e06fc
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 04:31:44 UTC; biocbuild
Author: Charlotte Soneson [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3833-2169>)
Maintainer: Charlotte Soneson <charlottesoneson@gmail.com>
