# Generated by roxygen2: do not edit by hand

export("corr<-")
export("deviation<-")
export("facetted<-")
export("fdrnbr<-")
export("fdrnbrcurve<-")
export("fdrtpr<-")
export("fdrtprcurve<-")
export("fpc<-")
export("fpr<-")
export("fsrnbr<-")
export("fsrnbrcurve<-")
export("maxsplit<-")
export("onlyshared<-")
export("overlap<-")
export("padj<-")
export("plotcolors<-")
export("pval<-")
export("roc<-")
export("scatter<-")
export("score<-")
export("splv<-")
export("sval<-")
export("tpr<-")
export("truth<-")
export(.COBRAData)
export(.COBRAPerformance)
export(.COBRAPlot)
export(COBRAData)
export(COBRAData_from_text)
export(COBRAData_to_text)
export(COBRAPerformance)
export(COBRAPlot)
export(COBRAapp)
export(basemethods)
export(calculate_adjp)
export(calculate_performance)
export(coerce)
export(corr)
export(deviation)
export(facetted)
export(fdrnbr)
export(fdrnbrcurve)
export(fdrtpr)
export(fdrtprcurve)
export(fpc)
export(fpr)
export(fsrnbr)
export(fsrnbrcurve)
export(maxsplit)
export(onlyshared)
export(overlap)
export(padj)
export(plot_corr)
export(plot_deviation)
export(plot_fdrnbrcurve)
export(plot_fdrtprcurve)
export(plot_fpc)
export(plot_fpr)
export(plot_fsrnbrcurve)
export(plot_overlap)
export(plot_roc)
export(plot_scatter)
export(plot_tpr)
export(plot_upset)
export(plotcolors)
export(prepare_data_for_plot)
export(pval)
export(reorder_levels)
export(roc)
export(scatter)
export(score)
export(splv)
export(stratiflevels)
export(sval)
export(tpr)
export(truth)
export(update_cobradata)
export(update_cobraperformance)
exportClasses(COBRAData)
exportClasses(COBRAPerformance)
exportClasses(COBRAPlot)
exportMethods("[")
exportMethods("corr<-")
exportMethods("deviation<-")
exportMethods("facetted<-")
exportMethods("fdrnbr<-")
exportMethods("fdrnbrcurve<-")
exportMethods("fdrtpr<-")
exportMethods("fdrtprcurve<-")
exportMethods("fpc<-")
exportMethods("fsrnbr<-")
exportMethods("fsrnbrcurve<-")
exportMethods("maxsplit<-")
exportMethods("onlyshared<-")
exportMethods("overlap<-")
exportMethods("padj<-")
exportMethods("plotcolors<-")
exportMethods("pval<-")
exportMethods("roc<-")
exportMethods("scatter<-")
exportMethods("score<-")
exportMethods("splv<-")
exportMethods("sval<-")
exportMethods("tpr<-")
exportMethods("truth<-")
exportMethods(basemethods)
exportMethods(corr)
exportMethods(deviation)
exportMethods(facetted)
exportMethods(fdrnbr)
exportMethods(fdrnbrcurve)
exportMethods(fdrtpr)
exportMethods(fdrtprcurve)
exportMethods(fpc)
exportMethods(fpr)
exportMethods(fsrnbr)
exportMethods(fsrnbrcurve)
exportMethods(maxsplit)
exportMethods(onlyshared)
exportMethods(overlap)
exportMethods(padj)
exportMethods(plotcolors)
exportMethods(pval)
exportMethods(roc)
exportMethods(scatter)
exportMethods(score)
exportMethods(splv)
exportMethods(stratiflevels)
exportMethods(sval)
exportMethods(tpr)
exportMethods(truth)
import(UpSetR)
import(dplyr)
import(ggplot2)
import(limma)
import(methods)
import(shiny)
import(shinydashboard)
importFrom(DT,datatable)
importFrom(ROCR,performance)
importFrom(ROCR,prediction)
importFrom(reshape2,melt)
importFrom(rlang,.data)
importFrom(scales,brewer_pal)
importFrom(scales,hue_pal)
importFrom(shinyBS,bsTooltip)
importFrom(utils,packageVersion)
