#' @rdname pval
#' @export
setGeneric("pval", function(x, ...) standardGeneric("pval"))
#' @rdname pval
#' @export
setGeneric("pval<-", function(x, ..., value) standardGeneric("pval<-"))

#' @rdname padj
#' @export
setGeneric("padj", function(x, ...) standardGeneric("padj"))
#' @rdname padj
#' @export
setGeneric("padj<-", function(x, ..., value) standardGeneric("padj<-"))

#' @rdname sval
#' @export
setGeneric("sval", function(x, ...) standardGeneric("sval"))
#' @rdname sval
#' @export
setGeneric("sval<-", function(x, ..., value) standardGeneric("sval<-"))

#' @rdname score
#' @export
setGeneric("score", function(x, ...) standardGeneric("score"))
#' @rdname score
#' @export
setGeneric("score<-", function(x, ..., value) standardGeneric("score<-"))

#' @rdname truth
#' @export
setGeneric("truth", function(x, ...) standardGeneric("truth"))
#' @rdname truth
#' @export
setGeneric("truth<-", function(x, ..., value) standardGeneric("truth<-"))


#' @rdname fdrtpr
#' @export
setGeneric("fdrtpr", function(x, ...) standardGeneric("fdrtpr"))
#' @rdname fdrtpr
#' @export
setGeneric("fdrtpr<-", function(x, ..., value) standardGeneric("fdrtpr<-"))

#' @rdname onlyshared
#' @export
setGeneric("onlyshared", function(x, ...) standardGeneric("onlyshared"))
#' @rdname onlyshared
#' @export
setGeneric("onlyshared<-", function(x, ..., value)
  standardGeneric("onlyshared<-"))

#' @rdname fdrtprcurve
#' @export
setGeneric("fdrtprcurve", function(x, ...) standardGeneric("fdrtprcurve"))
#' @rdname fdrtprcurve
#' @export
setGeneric("fdrtprcurve<-", function(x, ..., value)
  standardGeneric("fdrtprcurve<-"))

#' @rdname fdrnbr
#' @export
setGeneric("fdrnbr", function(x, ...) standardGeneric("fdrnbr"))
#' @rdname fdrnbr
#' @export
setGeneric("fdrnbr<-", function(x, ..., value) standardGeneric("fdrnbr<-"))

#' @rdname fdrnbrcurve
#' @export
setGeneric("fdrnbrcurve", function(x, ...) standardGeneric("fdrnbrcurve"))
#' @rdname fdrnbrcurve
#' @export
setGeneric("fdrnbrcurve<-", function(x, ..., value)
  standardGeneric("fdrnbrcurve<-"))

#' @rdname fsrnbr
#' @export
setGeneric("fsrnbr", function(x, ...) standardGeneric("fsrnbr"))
#' @rdname fsrnbr
#' @export
setGeneric("fsrnbr<-", function(x, ..., value) standardGeneric("fsrnbr<-"))

#' @rdname fsrnbrcurve
#' @export
setGeneric("fsrnbrcurve", function(x, ...) standardGeneric("fsrnbrcurve"))
#' @rdname fsrnbrcurve
#' @export
setGeneric("fsrnbrcurve<-", function(x, ..., value)
  standardGeneric("fsrnbrcurve<-"))

#' @rdname scatter
#' @export
setGeneric("scatter", function(x, ...) standardGeneric("scatter"))
#' @rdname scatter
#' @export
setGeneric("scatter<-", function(x, ..., value) standardGeneric("scatter<-"))

#' @rdname tpr
#' @export
setGeneric("tpr", function(x, ...) standardGeneric("tpr"))
#' @rdname tpr
#' @export
setGeneric("tpr<-", function(x, ..., value) standardGeneric("tpr<-"))

#' @rdname fpr
#' @export
setGeneric("fpr", function(x, ...) standardGeneric("fpr"))
#' @rdname fpr
#' @export
setGeneric("fpr<-", function(x, ..., value) standardGeneric("fpr<-"))

#' @rdname roc
#' @export
setGeneric("roc", function(x, ...) standardGeneric("roc"))
#' @rdname roc
#' @export
setGeneric("roc<-", function(x, ..., value) standardGeneric("roc<-"))

#' @rdname fpc
#' @export
setGeneric("fpc", function(x, ...) standardGeneric("fpc"))
#' @rdname fpc
#' @export
setGeneric("fpc<-", function(x, ..., value) standardGeneric("fpc<-"))

#' @rdname corr
#' @export
setGeneric("corr", function(x, ...) standardGeneric("corr"))
#' @rdname corr
#' @export
setGeneric("corr<-", function(x, ..., value) standardGeneric("corr<-"))

#' @rdname overlap
#' @export
setGeneric("overlap", function(x, ...) standardGeneric("overlap"))
#' @rdname overlap
#' @export
setGeneric("overlap<-", function(x, ..., value) standardGeneric("overlap<-"))

#' @rdname deviation
#' @export
setGeneric("deviation", function(x, ...) standardGeneric("deviation"))
#' @rdname deviation
#' @export
setGeneric("deviation<-", function(x, ..., value)
  standardGeneric("deviation<-"))

#' @rdname splv
#' @export
setGeneric("splv", function(x, ...) standardGeneric("splv"))
#' @rdname splv
#' @export
setGeneric("splv<-", function(x, ..., value) standardGeneric("splv<-"))

#' @rdname maxsplit
#' @export
setGeneric("maxsplit", function(x, ...) standardGeneric("maxsplit"))
#' @rdname maxsplit
#' @export
setGeneric("maxsplit<-", function(x, ..., value) standardGeneric("maxsplit<-"))

#' @rdname basemethods
#' @export
setGeneric("basemethods", function(x, ...) standardGeneric("basemethods"))

#' @rdname stratiflevels
#' @export
setGeneric("stratiflevels", function(x, ...) standardGeneric("stratiflevels"))


#' @rdname plotcolors
#' @export
setGeneric("plotcolors", function(x, ...) standardGeneric("plotcolors"))

#' @rdname plotcolors
#' @export
setGeneric("plotcolors<-", function(x, ..., value)
  standardGeneric("plotcolors<-"))

#' @rdname facetted
#' @export
setGeneric("facetted", function(x, ...) standardGeneric("facetted"))

#' @rdname facetted
#' @export
setGeneric("facetted<-", function(x, ..., value) standardGeneric("facetted<-"))
