## About iCOBRA

`iCOBRA` was developed in the group of Mark Robinson at 
the Institute for Molecular Life Sciences of the 
University of Zurich. 

### Developers

Charlotte Soneson [email](mailto:charlotte.soneson@imls.uzh.ch)

Mark Robinson [email](mailto:mark.robinson@imls.uzh.ch)

### Instruction video

An instruction video is available on 
[YouTube](https://www.youtube.com/watch?v=KpYCI31iQd4&feature=youtu.be)

### Benchmarking data sets

We have collected a set of benchmarking data sets, each 
consisting of (raw and/or processed) data and the true status 
for each feature. The collection contains benchmarking data sets 
for both statistical inference (differential isoform usage, 
differential gene expression) and target value estimation 
(transcript abundance estimation), and can be reached 
<a href="http://imlspenticton.uzh.ch/robinson_lab/benchmark_collection/" target="_blank">here</a>. 

### Code

All code for `iCOBRA` is available on 
<a href="https://github.com/markrobinsonuzh/iCOBRA" target="_blank">GitHub</a> and via <a href="http://bioconductor.org/packages/iCOBRA/" target="_blank">Bioconductor</a>.

### Citation

C Soneson & MD Robinson: iCOBRA: open, reproducible, standardized and live method benchmarking. *Nature Methods* 13:283 (2016).
